////////////////////////////////////////////////////////////////////////////////
// AReflection Library
// Copyright (c) 2004 by Arne Adams
// Permission to use, copy, modify, distribute and sell this software for any 
//     purpose is hereby granted without fee, provided that the above copyright 
//     notice appear in all copies and that both that copyright notice and this 
//     permission notice appear in supporting documentation.
// The author makes no representations about the suitability of this software 
//     for any purpose. It is provided "as is" without express or implied 
//     warranty.
////////////////////////////////////////////////////////////////////////////////
#ifndef REFLIB_QUALIFIEDTYPE_H_INC_
#define REFLIB_QUALIFIEDTYPE_H_INC_

#include <boost/pending/ct_if.hpp>

#include "boost/type_traits/is_const.hpp"

#include "boost/type_traits/add_const.hpp"

#include "boost/type_traits/add_reference.hpp"

#include "boost/type_traits/is_volatile.hpp"

#include "boost/type_traits/add_volatile.hpp"

#include <boost/type_traits/is_pointer.hpp>

namespace refl
{
    // assume that DependentType is a type of a member in EnclosingClass
    // then DependentType has to have the same cv qualification that EnclosingClass 
    // has.
    // This allows to use the same template for const and non-const classes
    template<class EnclosingClass, class DependentType> 
    struct GetQualifiedMemberType
    {
        enum {contextIsConst = boost::is_const<EnclosingClass>::value};  
        typedef typename boost::ct_if<(contextIsConst != 0),
                                    typename boost::add_const<DependentType>::type,
                                    DependentType>::type
        const_correct_type;
        enum {contextIsVol = boost::is_volatile<EnclosingClass>::value};  
        typedef typename boost::ct_if<(contextIsVol != 0),
                            typename boost::add_volatile<const_correct_type>::type,
                            const_correct_type>::type
        type;
        typedef typename ::boost::add_reference<type>::type  ref_type;
    };

    // facilitates static overloading
    struct TypeIsConstTag{};

    struct TypeIsNotConstTag{};

    template<class T> struct IsConst
    {
        enum {value = boost::is_const<T>::value};  
        typedef typename boost::ct_if<(value!= 0),
                                    TypeIsConstTag,
                                    TypeIsNotConstTag>::type
        type;
    };

    // facilitates static overloading again
    struct TypeIsPointerTag{};

    struct TypeIsNoPointerTag{};

    template<class T> struct IsPointer
    {
        enum {value = boost::is_pointer<T>::value};  
        typedef typename boost::ct_if<(value!= 0),
                                    TypeIsPointerTag,
                                    TypeIsNoPointerTag>::type
        type;
    };

    template<class T> struct Ref
    {
        typedef typename boost::add_reference<T>::type type;
    };
}
#endif 


